@@ -1,7 +1,123 @@
+use std::convert::TryFrom;
 use std::io::{Read, Seek};
 
 use crate::resource_value::ResourceValue;
-use crate::{read_u16, read_u32, ChunkHeader, ParseError};
+use crate::stringpool::StringPool;
+use crate::{read_u16, read_u32, ParseError};
+use num_enum::TryFromPrimitive;
+
+pub(crate) struct BinaryXmlDocument {
+    pub(crate) elements: Vec<XmlElement>,
+    pub(crate) string_pool: StringPool,
+    pub(crate) resource_map: Vec<u32>,
+}
+
+impl BinaryXmlDocument {
+    pub(crate) fn read_from_file<F: Read + Seek>(input: &mut F) -> Result<Self, ParseError> {
+        let header = ChunkHeader::read_from_file(input)?;
+
+        if header.typ != ResourceType::Xml {
+            return Err(ParseError::InvalidFile);
+        }
+
+        let mut elements = Vec::new();
+        let mut string_pool = None;
+        let mut resource_map = None;
+
+        loop {
+            let header = ChunkHeader::read_from_file(input);
+            if let Err(ParseError::IoError(_)) = &header {
+                break;
+            }
+            let header = header?;
+
+            match header.typ {
+                ResourceType::StringPool => {
+                    string_pool = Some(StringPool::read_from_file(input, &header)?);
+                }
+                ResourceType::XmlResourceMap => {
+                    resource_map = Some(parse_resource_map(input, &header)?);
+                }
+                ResourceType::XmlStartNameSpace => {
+                    elements.push(XmlElement::XmlStartNameSpace(
+                        XmlStartNameSpace::read_from_file(input, &header)?,
+                    ));
+                }
+                ResourceType::XmlEndNameSpace => {
+                    elements.push(XmlElement::XmlEndNameSpace(
+                        XmlEndNameSpace::read_from_file(input, &header)?,
+                    ));
+                }
+                ResourceType::XmlStartElement => {
+                    elements.push(XmlElement::XmlStartElement(
+                        XmlStartElement::read_from_file(input, &header)?,
+                    ));
+                }
+                ResourceType::XmlEndElement => {
+                    elements.push(XmlElement::XmlEndElement(XmlEndElement::read_from_file(
+                        input, &header,
+                    )?));
+                }
+                ResourceType::XmlCdata => {
+                    elements.push(XmlElement::XmlCdata(XmlCdata::read_from_file(
+                        input, &header,
+                    )?));
+                }
+                _ => return Err(ParseError::InvalidFile),
+            }
+        }
+
+        Ok(Self {
+            elements,
+            string_pool: string_pool.ok_or(ParseError::MissingStringPoolChunk)?,
+            resource_map: resource_map.ok_or(ParseError::MissingResourceMapChunk)?,
+        })
+    }
+}
+
+#[repr(u16)]
+#[derive(Debug, PartialEq, Clone, TryFromPrimitive)]
+pub(crate) enum ResourceType {
+    NullType = 0x000,
+    StringPool = 0x0001,
+    Table = 0x0002,
+    Xml = 0x0003,
+    XmlStartNameSpace = 0x0100,
+    XmlEndNameSpace = 0x101,
+    XmlStartElement = 0x0102,
+    XmlEndElement = 0x0103,
+    XmlCdata = 0x0104,
+    XmlLastChunk = 0x017f,
+    XmlResourceMap = 0x0180,
+    TablePackage = 0x0200,
+    TableType = 0x0201,
+    TableTypeSpec = 0x0202,
+    TableLibrary = 0x0203,
+}
+
+#[repr(C)]
+#[derive(Clone, Debug)]
+pub(crate) struct ChunkHeader {
+    pub(crate) typ: ResourceType,
+    pub(crate) header_size: u16,
+    pub(crate) size: u32,
+}
+
+impl ChunkHeader {
+    pub(crate) fn read_from_file<F: Read + Seek>(input: &mut F) -> Result<Self, ParseError> {
+        let typ = ResourceType::try_from(read_u16(input)?).map_err(|_| ParseError::InvalidFile)?;
+        let header_size = read_u16(input)?;
+        let size = read_u32(input)?;
+
+        let header = ChunkHeader {
+            typ,
+            header_size,
+            size,
+        };
+
+        Ok(header)
+    }
+}
 
 #[allow(clippy::enum_variant_names)]
 #[derive(Debug)]
@@ -13,6 +129,20 @@ pub(crate) enum XmlElement {
     XmlCdata(XmlCdata),
 }
 
+pub(crate) fn parse_resource_map<F: Read + Seek>(
+    input: &mut F,
+    header: &ChunkHeader,
+) -> Result<Vec<u32>, ParseError> {
+    let id_count = (header.size - u32::from(header.header_size)) / 4;
+
+    let mut ids = Vec::with_capacity(usize::try_from(id_count).unwrap());
+    for _ in 0..id_count {
+        ids.push(read_u32(input)?);
+    }
+
+    Ok(ids)
+}
+
 #[derive(Debug)]
 pub(crate) struct XmlNodeHeader {
     pub(crate) chunk_header: ChunkHeader,
@@ -61,7 +191,6 @@ impl XmlStartNameSpace {
             uri,
         };
 
-        // println!("{:?}", node);
         Ok(node)
     }
 }
@@ -88,7 +217,6 @@ impl XmlEndNameSpace {
             uri,
         };
 
-        // println!("{:?}", node);
         Ok(node)
     }
 }
@@ -182,7 +310,6 @@ impl XmlStartElement {
             attributes,
         };
 
-        // println!("{:?}", node);
         Ok(node)
     }
 }
@@ -205,7 +332,6 @@ impl XmlEndElement {
 
         let node = Self { header, ns, name };
 
-        // println!("{:?}", node);
         Ok(node)
     }
 }
@@ -232,7 +358,6 @@ impl XmlCdata {
             typed_data,
         };
 
-        // println!("{:?}", node);
         Ok(node)
     }
 }
